/* eslint-disable react-hooks/rules-of-hooks */
import React, { useState, useEffect } from "react"
import { isArray } from "lodash"
import {
    MapContainer,
    LayersControl,
    WMSTileLayer,
    GeoJSON,
} from "react-leaflet"
import "leaflet/dist/leaflet.css"

import MapsBaseLayer from "./map/MapsBaseLayer"
import MapstoreBaseLayer from "./map/MapstoreBaseLayer"

interface Props {
    mapid: number
    format?: "mapstore" | "maps"
}

const Map = (props: Props) => {
    if (typeof window !== "undefined") {
        let className = "z-40 h-full w-full"

        // Default id map (from mapstore)
        const { mapid: mapId, format = "maps" } = props

        // Default center, zoom
        const center: number[] = [47.1, -1.25]
        const zoom: number = 9
        const layers: any[] = []

        // Get current map Object when ready (whenCreated)
        const [map, setMap] = useState(null)

        // Init default map context
        const [mapContext, setMapContext] = useState({
            center: {
                x: center[1],
                y: center[0],
            },
            zoom: zoom,
            layers: layers,
        })

        // Get Mapstore context
        useEffect(() => {
            fetch(getContextUrl(format, mapId))
                .then((response) => response.json())
                .then((resultData) => {
                    let context = isFormatMaps(format)
                        ? formatContextMaps(resultData.config)
                        : resultData.map

                    setMapContext(context)

                    // Update map view with context data
                    if (map)
                        // @ts-ignore
                        map.setView(
                            [context.center.y, context.center.x],
                            context.zoom
                        )
                })
                .catch((error) => {
                    console.log(error)
                })
        }, [map, mapId])

        return (
            <>
                <MapContainer
                    center={[mapContext.center.y, mapContext.center.x]}
                    zoom={mapContext.zoom}
                    // @ts-ignore
                    whenCreated={setMap}
                    className={className}
                    // scrollWheelZoom={false}
                >
                    <LayersControl>
                        {
                            // BACKGROUNDS
                            isFormatMaps(format) ? (
                                <MapsBaseLayer layers={mapContext.layers} />
                            ) : (
                                <MapstoreBaseLayer layers={mapContext.layers} />
                            )
                        }
                        {
                            // OVERLAYS

                            // Load layers from context
                            mapContext.layers.map((layer, key) => {
                                if (isArray(layer.url)) return null
                                if (isFormatMaps(format)) {
                                    layer = formatLayerMaps(layer)
                                }

                                switch (layer.type?.toLowerCase()) {
                                    case "vector":
                                        return (
                                            <LayersControl.Overlay
                                                key={key}
                                                checked={layer.visibility}
                                                name={layer.name}
                                            >
                                                <GeoJSON
                                                    data={layer.features}
                                                    pathOptions={
                                                        layer.style || ""
                                                    }
                                                />
                                            </LayersControl.Overlay>
                                        )
                                    case "wfs":
                                        break
                                    case "wms":
                                        return (
                                            <LayersControl.Overlay
                                                key={key}
                                                checked={layer.visibility}
                                                name={layer.title}
                                            >
                                                <WMSTileLayer
                                                    layers={layer.name}
                                                    format={layer.format}
                                                    transparent={true}
                                                    url={layer.url}
                                                    styles={layer.style || ""}
                                                    opacity={layer.opacity}
                                                />
                                            </LayersControl.Overlay>
                                        )
                                }
                                return null
                            })
                        }
                    </LayersControl>
                </MapContainer>
            </>
        )
    }

    return null
}

const isFormatMaps = (format: string) => {
    return format == "maps"
}

const getContextUrl = (format: string, id: number) => {
    let url

    if (isFormatMaps(format)) {
        url = `/fr/maps/maps/${id}/`
    } else {
        url = `/geoportal/rest/geostore/data/${id}`
    }

    return url
}

const formatContextMaps = (oContext: any) => {
    const aCoordsCenter = oContext.coords
    oContext["center"] = { x: aCoordsCenter[0], y: aCoordsCenter[1] }
    return oContext
}

const formatLayerMaps = (oLayer: any) => {
    oLayer.visibility = oLayer.visible
    oLayer.name = oLayer.layername
    oLayer.style = ""
    oLayer.format = "image/png"
    return oLayer
}

export default Map
